@@ -54,7 +54,7 @@ use frame_metadata::{
     RuntimeMetadataPrefixed,
     StorageEntryType,
 };
-use heck::SnakeCase as _;
+use heck::ToSnakeCase as _;
 use proc_macro2::TokenStream as TokenStream2;
 use proc_macro_error::abort_call_site;
 use quote::{
@@ -147,6 +147,12 @@ impl RuntimeGenerator {
                 "frame_support::traits::misc::WrapperKeepOpaque",
                 parse_quote!(::subxt::WrapperKeepOpaque),
             ),
+            // BTreeMap and BTreeSet impose an `Ord` constraint on their key types. This
+            // can cause an issue with generated code that doesn't impl `Ord` by default.
+            // Decoding them to Vec by default (KeyedVec is just an alias for Vec with
+            // suitable type params) avoids these issues.
+            ("BTreeMap", parse_quote!(::subxt::KeyedVec)),
+            ("BTreeSet", parse_quote!(::std::vec::Vec)),
         ]
         .iter()
         .map(|(path, substitute): &(&str, syn::TypePath)| {
@@ -209,6 +215,7 @@ impl RuntimeGenerator {
 
             quote! {
                 pub mod #mod_name {
+                    use super::root_mod;
                     use super::#types_mod_ident;
                     #calls
                     #event
@@ -258,9 +265,8 @@ impl RuntimeGenerator {
                     pallet.calls.as_ref().map(|_| pallet_mod_name)
                 });
 
-        let error_details = errors::generate_error_details(&self.metadata);
-        let error_type = error_details.type_def;
-        let error_fn = error_details.dispatch_error_impl_fn;
+        let has_module_error_impl =
+            errors::generate_has_module_error_impl(&self.metadata, types_mod_ident);
 
         let default_account_data_ident = format_ident!("DefaultAccountData");
         let default_account_data_impl = generate_default_account_data_impl(
@@ -275,18 +281,17 @@ impl RuntimeGenerator {
         quote! {
             #[allow(dead_code, unused_imports, non_camel_case_types)]
             pub mod #mod_ident {
+                // Make it easy to access the root via `root_mod` at different levels:
+                use super::#mod_ident as root_mod;
+
                 #outer_event
                 #( #modules )*
                 #types_mod
 
                 /// The default error type returned when there is a runtime issue.
-                pub type DispatchError = self::runtime_types::sp_runtime::DispatchError;
-
-                // Statically generate error information so that we don't need runtime metadata for it.
-                #error_type
-                impl DispatchError {
-                    #error_fn
-                }
+                pub type DispatchError = #types_mod_ident::sp_runtime::DispatchError;
+                // Impl HasModuleError on DispatchError so we can pluck out module error details.
+                #has_module_error_impl
 
                 #default_account_data_impl
 
@@ -323,12 +328,33 @@ impl RuntimeGenerator {
                     pub fn tx(&'a self) -> TransactionApi<'a, T, X, A> {
                         TransactionApi { client: &self.client, marker: ::core::marker::PhantomData }
                     }
+
+                    pub fn events(&'a self) -> EventsApi<'a, T> {
+                        EventsApi { client: &self.client }
+                    }
+                }
+
+                pub struct EventsApi<'a, T: ::subxt::Config> {
+                    client: &'a ::subxt::Client<T>,
+                }
+
+                impl <'a, T: ::subxt::Config> EventsApi<'a, T> {
+                    pub async fn at(&self, block_hash: T::Hash) -> Result<::subxt::events::Events<'a, T, Event>, ::subxt::BasicError> {
+                        ::subxt::events::at::<T, Event>(self.client, block_hash).await
+                    }
+
+                    pub async fn subscribe(&self) -> Result<::subxt::events::EventSubscription<'a, T, Event>, ::subxt::BasicError> {
+                        ::subxt::events::subscribe::<T, Event>(self.client).await
+                    }
+
+                    pub async fn subscribe_finalized(&self) -> Result<::subxt::events::EventSubscription<'a, T, Event>, ::subxt::BasicError> {
+                        ::subxt::events::subscribe_finalized::<T, Event>(self.client).await
+                    }
                 }
 
                 pub struct ConstantsApi;
 
-                impl ConstantsApi
-                {
+                impl ConstantsApi {
                     #(
                         pub fn #pallets_with_constants(&self) -> #pallets_with_constants::constants::ConstantsApi {
                             #pallets_with_constants::constants::ConstantsApi
@@ -417,7 +443,8 @@ fn generate_default_account_data_impl(
         };
 
     // this path to the storage entry depends on storage codegen.
-    let storage_entry_path = quote!(self::system::storage::Account);
+    // AccountOwned contains the same data as Account does, but without references.
+    let storage_entry_path = quote!(self::system::storage::AccountOwned);
 
     Some(quote! {
         /// The default storage entry from which to fetch an account nonce, required for
@@ -429,12 +456,12 @@ fn generate_default_account_data_impl(
             type AccountId = #account_id_ty;
             type Index = #account_nonce_ty;
 
-            fn nonce(result: &<Self::StorageEntry as ::subxt::StorageEntry>::Value) -> Self::Index {
-                result.nonce
-            }
             fn storage_entry(account_id: Self::AccountId) -> Self::StorageEntry {
                 #storage_entry_path(account_id)
             }
+            fn nonce(result: &<Self::StorageEntry as ::subxt::StorageEntry>::Value) -> Self::Index {
+                result.nonce
+            }
         }
     })
 }
@@ -462,7 +489,7 @@ where
                     type_gen,
                 );
                 CompositeDef::struct_def(
-                    var.name(),
+                    struct_name.as_ref(),
                     Default::default(),
                     fields,
                     Some(parse_quote!(pub)),
