@@ -25,5 +25,9 @@ pub use self::{
         generate_runtime_api,
         RuntimeGenerator,
     },
-    types::GeneratedTypeDerives,
+    types::{
+        GeneratedTypeDerives,
+        Module,
+        TypeGenerator,
+    },
 };
